﻿using System;
using System.Collections.Generic;
using System.Linq;

using DevExpress.XtraPivotGrid;

namespace TestProject
{
    /// <summary>
    /// 커스텀 SUMMARY 헬퍼
    /// </summary>
    public static class CustomSummaryHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 그룹 SUMMARY 값 구하기 - GetGroupSummaryValue(groupingEnumerable, target, dataFieldName)

        /// <summary>
        /// 그룹 SUMMARY 값 구하기
        /// </summary>
        /// <param name="groupingEnumerable">그룹핑 열거 가능형</param>
        /// <param name="target">타겟</param>
        /// <param name="dataFieldName">데이터 필드명</param>
        /// <returns>그룹 SUMMARY 값</returns>
        public static decimal GetGroupSummaryValue
        (
            IEnumerable<IGrouping<object, PivotDrillDownDataRow>> groupingEnumerable,
            object                                                target,
            string                                                dataFieldName
        )
        {
            var targetGroup = groupingEnumerable.FirstOrDefault(g => object.Equals(g.Key, target));

            if(targetGroup == null)
            {
                return 0;
            }

            return targetGroup.Sum(r => Convert.ToDecimal(r[dataFieldName]));
        }

        #endregion
        #region 커스텀 값 계산 강제 여부 구하기 - ShouldCalculateCustomValue(groupField, e)

        /// <summary>
        /// 커스텀 값 계산 강제 여부 구하기
        /// </summary>
        /// <param name="groupField">그룹 필드</param>
        /// <param name="e">이벤트 인자</param>
        /// <returns>커스텀 값 계산 강제 여부</returns>
        public static bool ShouldCalculateCustomValue(PivotGridField groupField, PivotGridCustomSummaryEventArgs e)
        {
            if(!groupField.Visible || groupField.Area == PivotArea.FilterArea || groupField.Area == PivotArea.DataArea)
            {
                return true;
            }

            if(groupField.Area == PivotArea.RowArea && (e.RowField == null || e.RowField.AreaIndex < groupField.AreaIndex))
            {
                return true;
            }

            if(groupField.Area == PivotArea.ColumnArea && (e.ColumnField == null || e.ColumnField.AreaIndex < groupField.AreaIndex))
            {
                return true;
            }

            return false;
        }

        #endregion
    }
}