using System;
using System.Linq;

using DevExpress.Data;
using DevExpress.Data.PivotGrid;
using DevExpress.XtraEditors;
using DevExpress.XtraPivotGrid;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainForm()

        /// <summary>
        /// 
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            SelectCustomSummaryApproach(this.pivotGridControl.OptionsData.DataProcessingEngine);

            this.optimizedModeToggleSwitch.Toggled      += optimizedModeToggleSwitch_Toggled;
            this.pivotGridControl.FieldValueDisplayText += pivotGridControl_FieldValueDisplayText;

            this.pivotGridControl.DataSource = TestData.GetTestDataList();

            this.pivotGridControl.BestFit();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region ȭ   ġ ۽ óϱ - optimizedModeToggleSwitch_Toggled(sender, e)

        /// <summary>
        /// ȭ   ġ ۽ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void optimizedModeToggleSwitch_Toggled(object sender, EventArgs e)
        {
            ToggleSwitch toggleSwitch = sender as ToggleSwitch;

            if(toggleSwitch.IsOn)
            {
                this.pivotGridControl.Fields["Value"].SummaryType = PivotSummaryType.Sum;

                this.pivotGridControl.CustomSummary -= pivotGridControl_CustomSummary;

                this.pivotGridControl.OptionsData.DataProcessingEngine = PivotDataProcessingEngine.Optimized;

                SelectCustomSummaryApproach(PivotDataProcessingEngine.Optimized);
            }
            else
            {
                this.pivotGridControl.OptionsData.DataProcessingEngine = PivotDataProcessingEngine.LegacyOptimized;

                SelectCustomSummaryApproach(PivotDataProcessingEngine.LegacyOptimized);
            };
        }

        #endregion
        #region ǹ ׸ Ʈ SUMMARY Ŀ ϱ - pivotGridControl_CustomSummary(sender, e)

        /// <summary>
        /// ǹ ׸ Ʈ SUMMARY Ŀ ϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void pivotGridControl_CustomSummary(object sender, PivotGridCustomSummaryEventArgs e)
        {
            if(e.DataField.Name == "fieldValue")
            {
                if(CustomSummaryHelper.ShouldCalculateCustomValue(fieldType, e))
                {
                    var groupedDataSource = e.CreateDrillDownDataSource().Cast<PivotDrillDownDataRow>().GroupBy(r => r[fieldType]);

                    decimal incomeSummary = CustomSummaryHelper.GetGroupSummaryValue(groupedDataSource, "", e.FieldName);
                    decimal outlaySummary = CustomSummaryHelper.GetGroupSummaryValue(groupedDataSource, "", e.FieldName);

                    e.CustomValue = incomeSummary - outlaySummary;
                }
                else
                {
                    e.CustomValue = e.SummaryValue.Summary;
                }
            }
        }

        #endregion
        #region ǹ ׸ Ʈ ʵ  ÷ ؽƮ ϱ - pivotGridControl_FieldValueDisplayText(sender, e)

        /// <summary>
        /// ǹ ׸ Ʈ ʵ  ÷ ؽƮ ϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void pivotGridControl_FieldValueDisplayText(object sender, PivotFieldDisplayTextEventArgs e)
        {
            if(e.ValueType == PivotGridValueType.GrandTotal)
            {
                if(e.IsColumn)
                {
                    e.DisplayText = "Ұ";
                }
                else
                {
                    e.DisplayText = "Ѱ";
                }
            }

            if(e.ValueType == PivotGridValueType.Total)
            {
                if(e.IsColumn)
                {
                    e.DisplayText = e.DisplayText.Replace("Total", "");
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region  Ұ ʵ ߰ϱ - AddValueTotalField()

        /// <summary>
        ///  Ұ ʵ ߰ϱ
        /// </summary>
        /// <returns> Ұ ʵ</returns>
        private PivotGridField AddValueTotalField()
        {
            PivotGridField field = new PivotGridField
            {
                Name      = "fieldValueTotal",
                FieldName = "Savings",
                Caption   = "",
                Area      = PivotArea.DataArea,
                AreaIndex = 0,
            };

            field.Options.ShowValues = false;

            this.pivotGridControl.Fields.Add(field);

            return field;
        }

        #endregion
        #region ȭ ٹ ϱ - UseOptimizedApproach()

        /// <summary>
        /// ȭ ٹ ϱ
        /// </summary>
        private void UseOptimizedApproach()
        {
            this.pivotGridControl.Fields["Value"].Options.ShowTotals     = false;
            this.pivotGridControl.Fields["Value"].Options.ShowGrandTotal = false;

            PivotGridField valueTotalField = this.pivotGridControl.Fields.GetFieldByName("fieldValueTotal");
            
            if(valueTotalField == null)
            {
                valueTotalField = AddValueTotalField();
            }

            ExpressionDataBinding savingsBinding = new ExpressionDataBinding("Sum(Iif([Type]='', [Value], -[Value]))");

            valueTotalField.DataBinding = savingsBinding;
        }

        #endregion
        #region ٿ  ٹ ϱ - UseUnboundExpressionApproach()

        /// <summary>
        /// ٿ  ٹ ϱ
        /// </summary>
        private void UseUnboundExpressionApproach()
        {
            this.pivotGridControl.Fields["Value"].Options.ShowTotals     = false;
            this.pivotGridControl.Fields["Value"].Options.ShowGrandTotal = false;

            PivotGridField valueTotalField = this.pivotGridControl.Fields.GetFieldByName("fieldValueTotal");
            
            if(valueTotalField == null)
            {
                valueTotalField = AddValueTotalField();
            }

            valueTotalField.UnboundExpression     = "Sum(Iif([Type]='', [Value], -[Value]))";
            valueTotalField.UnboundType           = UnboundColumnType.Decimal;
            valueTotalField.UnboundExpressionMode = UnboundExpressionMode.UseAggregateFunctions;
        }

        #endregion
        #region Ž ٹ ϱ - UseLegacyApproach()

        /// <summary>
        /// Ž ٹ ϱ
        /// </summary>
        private void UseLegacyApproach()
        {
            this.pivotGridControl.Fields["Value"].SummaryType = PivotSummaryType.Custom;

            this.pivotGridControl.CustomSummary += pivotGridControl_CustomSummary;
        }

        #endregion
        #region Ŀ SUMMARY ٹ ϱ - SelectCustomSummaryApproach(processingEngine)

        /// <summary>
        /// Ŀ SUMMARY ٹ ϱ
        /// </summary>
        /// <param name="processingEngine">ǹ  ó </param>
        private void SelectCustomSummaryApproach(PivotDataProcessingEngine processingEngine)
        {
            switch(processingEngine)
            {
                case PivotDataProcessingEngine.Optimized       : UseOptimizedApproach();         break;
                case PivotDataProcessingEngine.LegacyOptimized : UseUnboundExpressionApproach(); break;
                default                                        : UseLegacyApproach();            break;
            }
        }

        #endregion
    }
}